<?php
/*===========================================================================*/
/*                      Secure CGI parameters parsing						*/
/*===========================================================================*/
/* (C) Seiros LLC													   2006 */
/*===========================================================================*/
// {{{ cgi_param ( )
function cmp_struint($a, $b)
{
	$a = trim($a);
	$b = trim($b);
	if( !ereg("^[0-9]+$", $a) )
		return null;
	if( !ereg("^[0-9]+$", $b) )
		return null;
	# Length test
	$al = strlen($a);
	$bl = strlen($b);
	if( $al < $bl )
		return -1;
	elseif( $al > $bl )
		return 1;

	if( substr($a, 0, 1) < substr($b, 0, 1) )
		return -1;
	elseif( substr($a, 0, 1) > substr($b, 0, 1) )
		return 1;
	elseif($al == 1)
		return 0;

	return cmp_struint(substr($a, 1), substr($b, 1));
}

function cmp_strint($a, $b)
{
	$a = trim($a);
	$b = trim($b);
	if( !ereg("^-?[0-9]+$", $a) )
		return null;
	if( !ereg("^-?[0-9]+$", $b) )
		return null;
	# Sign
	$as = substr($a, 0, 1) == '-'? -1: 1;
	$bs = substr($b, 0, 1) == '-'? -1: 1;
	if( $as < $bs )
		return -1;
	elseif( $as > $bs )
		return 1;
	elseif( $as < 0 )
	{
		$sign = -1;
		$a = trim($a, '-');
		$b = trim($b, '-');
	}
	elseif( $bs > 0 )
		$sign = 1;
	else
		die("Unrecoverable error in cmp_strint\n");

	$al = strlen($a);
	$bl = strlen($b);
	if( $al < $bl )
		return -$sign;
	elseif( $al > $bl )
		return $sign;

	return $sign * strcmp(trim($a, '-'), trim($b, '-'));
}

/**
 * Convert var to specified type with strict data test
 */
function to_type($var, $type)
{
	if( $var === null )
		return null;
	switch ( $type ) {
	case 'num':
		$var = str_replace(",", ".", $var);
		$var = trim($var);
		if( $var == '.' )
			return null;
		if( !ereg("^-?[0-9]*\.?[0-9]*$", $var ) )
			return null;
		if( $var == "." )
			return null;
		if( $var == "-." )
			return null;
		return (double)$var;
	case 'int':
		if( !ereg("^[[:space:]]*-?[0-9]+[[:space:]]*$", $var ) )
			return null;
		if( cmp_strint('2147483647', $var) < 0 )
			return null;
		if( cmp_strint('-2147483648', $var) > 0 )
			return null;
		return (integer)$var;
	case 'str':
		if( strpos($var, "\0") )
			return null;
		if( strpos($var, "\x1b") )
			return null;
		$var = trim($var);
		if( ereg("[\x01-\x08\x0a-\x1f]", $var) )
			return null;
	case 'text':
		if( strpos($var, "\0") )
			return null;
		if( strpos($var, "\x1b") )
			return null;
		return (string)$var;
	default:
		return null;
	}
}

function limit_to_type($limit, $type)
{
	switch( $type )
	{
	case 'str':
	case 'text':
	case 'int':
		return to_type($limit, 'int');
	case 'num':
		return to_type($limit, 'num');
	}
	return null;
}

function var_constraint( $var, $constraints )
{
	$type = isset($constraints["type"])? $constraints["type"]: "str";
	$default = null;
	switch( $type )
	{
	case "int":
	case "num":
	case "str":
	case "text":
		break;
	default:
		return null;
	}
	if ( isset( $constraints["default"] ) )
	{
		$default = to_type($constraints["default"], $type);
		if( $default === null )
			return null;
	}

	if(  ($var = to_type($var, $type)) === null )
		return $default;

	if( isset( $constraints['min'] ) )
	{
		$constraints['min'] = limit_to_type($constraints['min'], $type);
		if( $constraints['min'] === null )
			return null;
	}
	if( isset( $constraints['max'] ) )
	{
		$constraints['max'] = limit_to_type($constraints['max'], $type);
		if( $constraints['max'] === null )
			return null;
	}

	if( $type == 'str' or $type == 'text' )
		$val = strlen($var);	/*************************/
	else						/* Data for max/min test */
		$val = $var;			/*************************/

	if( $var === null
		or
		(isset( $constraints['max'] ) and $val > $constraints['max'] )
		or
		(isset( $constraints['min'] ) and $val < $constraints['min'] )
		or
		(isset( $constraints['regex'] ) and !ereg( $constraints['regex'], $var ) ))
		{
				return $default;
		}
    return $var;				
}

/**
 * @access public
 *
 * @param array(array()...array());
 *
 *		$array( 'name'	=> $name,
 *				'default' => $def_val,
 *				'type'	=> $type
 *			   )
 *
 * @return array();
 */
function cgi_param( $array, $inbound = null )
{
	/**
	 * bad param
	 */
	if ( empty( $array ) or ( ! is_array( $array ) ) )
		return null;

	if( $inbound === null )
		$inbound = $_REQUEST;

	$rc = array();
	foreach( $array as $key => $value )
	{
		$name = isset($value[ 'name' ])? $value[ 'name' ]: $key;
		$type = isset($value["type"])? $value["type"]: "str";
		
		if ( isset( $value["default"] ) )
		{
			$default = to_type($value["default"], $type);
			if( $default === null )
				continue;
		} else {
			unset($default);
		}


		// If variable not set -- return default value if exists
		if ( !isset( $inbound[ $name ] ) ) {
			if ( isset( $default ) )
				$rc[ $name ] = $default;
			continue;
		}

		$tmp = var_constraint( $inbound[ $name ], $value );

		if( $tmp !== null )
			$rc [ $name ] = $tmp;
	} // foreach
	return $rc;
}
?>
